
#ifndef __processManager__
#define __processManager__

#include <yarp/os/RateThread.h>

#include <memoryManager.h>
#include <displayManager.h>


class processManager : public yarp::os::RateThread
{
protected:
    memoryManager &mmu;
    unsigned char *phyMem;

    displayManager *showMMU;
    displayManager *showPHY;
    displayManager *showDIF;

    unsigned int   basePhyAddr;
    unsigned char *curVirtualAddr;

    double logT;
    double t0;

    /**
     * genVirtualAddress()
     * - generates a random virtual address using a gaussian 
     *   distribution over the whole memory space
     */
    unsigned char *genVirtualAddress();

    /**
     * getEqPhyscalAddress()
     * - given a virtual address va, returns the corresponding 
     *   physical address of the concurrent memory
     */
    unsigned char *getEqPhyscalAddress(unsigned char * const va);

    /**
     * updateDisplay()
     * - updates the internal memory images for logging purpose
     */
    void updateDisplay(unsigned char *addr, const unsigned char c1, const unsigned char c2);

public:
    processManager(const int mainPeriod, const int logPeriod, memoryManager &_mmu);

    virtual bool threadInit();
    virtual void threadRelease();
    virtual void run();
};

#endif
