
#include <processManager.h>
#include <myheader.h>

#include <memory.h>

#include <yarp/os/Time.h>
#include <yarp/os/Random.h>

using namespace yarp;
using namespace yarp::os;
using namespace std;


processManager::processManager(const int mainPeriod, const int logPeriod, memoryManager &_mmu) : 
                RateThread(mainPeriod), mmu(_mmu)
{
    phyMem=NULL;
    showMMU=showPHY=showDIF=NULL;

    logT=logPeriod/1000.0;
}


bool processManager::threadInit()
{
    // allocate concurrent physical memory
    phyMem=new unsigned char[ADDRESS_SPACE];
    memset(phyMem,0,ADDRESS_SPACE); // clean it up

    // allocate other useful stuff ...
    showMMU=new displayManager("/mmu");
    showPHY=new displayManager("/phy");
    showDIF=new displayManager("/dif");

    basePhyAddr=int(&phyMem[0]);
    curVirtualAddr=(unsigned char*)(ADDRESS_SPACE>>1);    

    t0=Time::now();
    Random::seed(0);

    return true;
}


void processManager::threadRelease()
{
    if (phyMem)
    {
        delete[] phyMem;
        phyMem=NULL;
    }

    if (showMMU)
    {
        delete showMMU;
        showMMU=NULL;
    }

    if (showPHY)
    {
        delete showPHY;
        showPHY=NULL;
    }

    if (showDIF)
    {
        delete showDIF;
        showDIF=NULL;
    }    
}


unsigned char *processManager::genVirtualAddress()
{
    // generate random address delta
    curVirtualAddr+=int(Random::normal(0.0,100.0));

    // deal with wrap around
    if (int(curVirtualAddr)>=ADDRESS_SPACE)
        curVirtualAddr-=ADDRESS_SPACE;
    else if (int(curVirtualAddr)<0)
        curVirtualAddr+=ADDRESS_SPACE;

    return curVirtualAddr;
}


inline unsigned char *processManager::getEqPhyscalAddress(unsigned char * const va)
{
    return va+basePhyAddr;
}


void processManager::updateDisplay(unsigned char *addr, const unsigned char c1, const unsigned char c2)
{
    const double t=Time::now();

    showMMU->set(int(addr),c1);
    showPHY->set(int(addr),c2);
    showDIF->set(int(addr),c1-c2);    

    if (t-t0>=logT)
    {
        showMMU->snapshot();
        showPHY->snapshot();
        showDIF->snapshot();

        t0=t;
    }
}


void processManager::run()
{
    for (int i=0; i<1000; i++)
    {
        unsigned char *va=genVirtualAddress();
        unsigned char *pa=getEqPhyscalAddress(va);

        (*mmu.writeAddress(va))++;  // access to virtual memory
        (*pa)++;                    // access to physical memory

        unsigned char c1=*mmu.readAddress(va);
        unsigned char c2=*pa;

        updateDisplay(va,c1,c2);
    }
}



